#!/usr/bin/env perl
use strict;


# multi_po.pl sk.po --add cs.po CS --add ru.po RU > sk2.po
# multi_po.pl sk2.po --remove > sk.po


use constant VERBOSE => 0;


package PO::String;


sub add_line {
	my $this = shift;
	my $line = shift;

	if ($line =~ m/^\# /) {

		$this->{"comment_code"} .= $line;

	} elsif ($line =~ m/^\#/) {

		$this->{"comment2_code"} .= $line;

	} elsif ($line =~ m/^msgid /) {

		if ("" ne $this->{"msgid_code"}) {
			croak("Duplicate MSGID in line: " . $line);
		}
		$this->{"msgid_code"} = $line;

	} elsif ($line =~ m/^msgstr /) {

		if ("" ne $this->{"msgstr_code"}) {
			croak("Duplicate MSGSTR in line: " . $line);
		}
		$this->{"msgstr_code"} = $line;

	} elsif ($line =~ m/^"/) {

		if ("" ne $this->{"msgstr_code"}) {
			$this->{"msgstr_code"} .= $line;
		} elsif ("" ne $this->{"msgid_code"}) {
			$this->{"msgid_code"} .= $line;
		} else {
			croak("String outside of MSGID or MSGSTR in line: " . $line);
		}

	} else {
		croak("Unexpected line: " . $line);
	}
}


sub add_translation {
	my $this = shift;
	my $hash = shift;
	my $language = shift;

	my $msgid_code = $this->{"msgid_code"};
	my $msgid = $this->msg_code_to_value($msgid_code);
	my $msgstr = %{$hash}->{$msgid};
	if (defined($msgstr)) {
		$this->{"comment_code"} .= "# { " . $language . " }\n";
		my @lines = split(/\\n/, $msgstr);
		foreach my $line (@lines) {
			$this->{"comment_code"} .= "# " . $line . "\n";
		}
	}
}


sub create_new {
	my $class = shift;

	my $this = {
	 "comment_code" => "",
	 "comment2_code" => "",
	 "msgid_code" => "",
	 "msgstr_code" => ""
	};
	bless $this, $class;
	return $this;
}


sub dump {
	my $this = shift;

	print $this->{"comment_code"};
	print $this->{"comment2_code"};
	print $this->{"msgid_code"};
	print $this->{"msgstr_code"};
	print "\n";
}


sub get_msgid {
	my $this = shift;
	my $msgid_code = $this->{"msgid_code"};
	my $msgid = $this->msg_code_to_value($msgid_code);
	return $msgid;
}


sub get_msgstr {
	my $this = shift;
	my $msgstr_code = $this->{"msgstr_code"};
	my $msgstr = $this->msg_code_to_value($msgstr_code);
	return $msgstr;
}


sub msg_code_to_value {
	my $this = shift;
	my $code = shift;

	my $value = "";
	my @lines = split(/\n/s, $code);
	foreach my $line (@lines) {
		($line =~ m/^\w*\s*"(.*)"$/) or croak("Incorrect MSG line: " . $line);
		$value .= $1;
	}
	return $value;
}


sub remove_translations {
	my $this = shift;

	$this->{"comment_code"} =~ s/^\# \{ \w+ \}.*$//s;
}


package PO::Catalog;


sub about {
	my $this = shift;

	my $string_count = scalar(@{$this->{"strings"}});
	return "Catalog (" . $string_count . " strings)\n";
}


sub add_translation {
	my $this = shift;
	my $catalog = shift;
	my $language = shift;

	my $hash = $catalog->to_hash();
	foreach my $string (@{$this->{"strings"}}) {
		$string->add_translation($hash, $language);
	}
}


sub add_string {
	my $this = shift;
	my $string = shift;

	push(@{$this->{"strings"}}, $string);
}


sub create_new {
	my $class = shift;
	my $file_path = shift;

	my $this = {
	 "strings" => []
	};
	bless $this, $class;
	return $this;
}


sub dump {
	my $this = shift;

	foreach my $string (@{$this->{"strings"}}) {
		$string->dump();
	}
}


sub remove_translations {
	my $this = shift;

	foreach my $string (@{$this->{"strings"}}) {
		$string->remove_translations();
	}
}


sub to_hash {
	my $this = shift;

	my $hash = {};
	foreach my $string (@{$this->{"strings"}}) {
		my $msgid = $string->get_msgid();
		my $msgstr = $string->get_msgstr();
		if ("" eq $msgid) {
			next;
		}
		%{$hash}->{$msgid} = $msgstr;
	}
	return $hash;
}


package PO::Reader;


use Carp;


sub read_from_file {
	my $file_path = shift;

	my $catalog = PO::Catalog->create_new();
	my $string = undef;
	my $file = undef;

	open($file, $file_path) or croak("Cannot open catalog file: \"" . $file_path . "\" for reading.\n");
	while (my $line = <$file>) {
		if ($line =~ m/^\s*$/) {

			if (!defined($string)) {
				next;
			}
			$catalog->add_string($string);
			$string = undef;

		} else {

			if (!defined($string)) {
				$string = PO::String->create_new();
			}
			$string->add_line($line);

		}
	}
	close($file);

	if (defined($string)) {
		$catalog->add_string($string);
	}
	return $catalog;
}


package main;


use Carp;


if (!@ARGV) {
	print "Usage:\n" .
	"multi_po <catalog> --add <catalog-2> <language-code-2> > <new-catalog>\n" .
	"or\n" .
	"multi_po <catalog> --remove > <new-catalog>\n";
	exit;
}

my $catalog_path = shift(@ARGV);
if (VERBOSE) {
	print "Base catalog: " . $catalog_path . "\n";
}
my $catalog = PO::Reader::read_from_file($catalog_path);

if (@ARGV) {
	my $command = shift(@ARGV);
	if ("--add" eq $command) {

		while(@ARGV) {
			my $catalog2_path = shift(@ARGV);
			my $language = shift(@ARGV);
			if (VERBOSE) {
				print "Add translation \"" . $language . "\" from catalog: " . $catalog2_path . "\n";
			}
			my $catalog2 = PO::Reader::read_from_file($catalog2_path);
			$catalog->add_translation($catalog2, $language);
		}


	} elsif ("--remove" eq $command) {

		if (VERBOSE) {
			print "Remove translations\n";
		}
		$catalog->remove_translations();

	} else {
		croak("Unknown command \"" . $command . "\".\n");
	}
}

$catalog->dump();
